import { createAudioPlayer, entersState, joinVoiceChannel, VoiceConnectionStatus, } from "@discordjs/voice";
import { StageChannel } from "discord.js";
import { sLogger } from "..";
var VoiceConstructError;
(function (VoiceConstructError) {
    VoiceConstructError[VoiceConstructError["success"] = 0] = "success";
    VoiceConstructError[VoiceConstructError["user_not_in_voice"] = 1] = "user_not_in_voice";
    VoiceConstructError[VoiceConstructError["is_stage_channel"] = 2] = "is_stage_channel";
})(VoiceConstructError || (VoiceConstructError = {}));
export class Voice {
    constructor(msg) {
        this.error = VoiceConstructError.success;
        this.player = createAudioPlayer();
        const voiceChannel = msg.member?.voice.channel;
        if (!voiceChannel) {
            this.error = VoiceConstructError.user_not_in_voice;
            return;
        }
        this.channelName = voiceChannel.name;
        if (voiceChannel instanceof StageChannel) {
            this.error = VoiceConstructError.is_stage_channel;
            return;
        }
        this.connection = joinVoiceChannel({
            channelId: voiceChannel.id,
            guildId: voiceChannel.guild.id,
            adapterCreator: voiceChannel.guild
                .voiceAdapterCreator,
            selfMute: false,
            selfDeaf: false,
        });
        this.connection.subscribe(this.player);
    }
    async waitTillReady() {
        if (!this.connection)
            return false;
        try {
            await entersState(this.connection, VoiceConnectionStatus.Ready, 5000);
            return true;
        }
        catch (err) {
            sLogger.log(`Error joining ${this.channelName} : ${err}`, "ERROR");
            return false;
        }
    }
}
